__author__     = "Aaron Straup Cope"
__url__        = "http://aaronland.info/python/w5"
__version__    = "1.0a7"
__cvsversion__ = "$Revision: 1.27 $"
__date__       = "$Date: 2004/11/02 21:50:23 $"
__copyright__  = "%s\n%s\n%s" % \
                 ("This application is licensed under the",
                  "Creative Commons Attribution-NonCommercial License",
                  "http://creativecommons.org/licenses/by-nc/1.0/")

__license__    = "This application is licensed under the Creative \
Commons Attribution-NonCommercial License. To view a copy of this \
license, visit http://creativecommons.org/licenses/by-nc/1.0/ or send \
a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, \
California 94305, USA."

# $Id: rConfig.py,v 1.27 2004/11/02 21:50:23 asc Exp $

from shutil  import copy
from os      import getuid,stat
from os.path import abspath,isfile
from pwd     import getpwuid

from rdflib.constants   import *
from rdflib.Literal	import Literal
from rdflib.Namespace   import Namespace
from rdflib.TripleStore import TripleStore
from rdflib.URIRef	import URIRef

from rConstants import NS_W5, W5_PREFIX, \
                       W5_SERVICE,W5_PROTOCOL,W5_LOGIN, W5_FILTER, \
                       W5_APP, W5_PREFS, \
                       NS_DUBLINCORE, DC_PREFIX, DC_TITLE, \
		       PROTOCOL_XMLRPC

from rService   import rService

class rConfig:

    def __init__ (self,file) :

        self.__store__     = TripleStore()
        self.__cfg__       = None
        self.__services__  = []
        self.__protocols__ = []
        self.__lastread__  = 0
        
        self.__ns__        = {W5_PREFIX:Namespace(NS_W5),
                              DC_PREFIX:Namespace(NS_DUBLINCORE)}       

	if not isfile(file) :
            self.__store__.save(file)

        self.__cfg__ = file

    #
    
    def __del__(self):
        self.Write()
        
    #
    
    def Load (self) :

        if self.__cfg__ == None:
            raise Exception, ERR_NOCONFIG

        #
        
        modified = stat(self.__cfg__)[8]
        
        if self.__lastread__ >= modified:
            return True

        #

        try :
            self.__store__.load(self.__cfg__)
        except Exception, inst:
            raise Exception, ERR_LOADCONFIG
        
        #

        self.__services__  = []
        self.__protocols__ = []
        
        for s in self.__store__.subjects(TYPE,self.QAttr(W5_SERVICE)):
            self.__services__.append(s)

        for s in self.__store__.subjects(TYPE,self.QAttr(W5_PROTOCOL)):
            self.__protocols__.append(s)

        #
        
        self.__lastread__ = modified
        return True

    #
    
    def Write (self) :

        # *cough cough cough*
        # An unfortunate hack until I
        # can satisfy myself that this
        # resource won't get accidentally
        # deleted.
        
        self.AddProtocol(PROTOCOL_XMLRPC,"XML-RPC")

        #
        
        file   = abspath(self.__cfg__)
        backup = file + ".bak"

        try:
            copy(file,backup)
        except Exception, inst:
            raise Exception, ERR_BACKUPCONFIG
        
        try:
            self.__store__.save(file)
            return True
        except Exception, inst:
            raise Exception, ERR_SAVECONFIG

    #
    
    def QAttr (self,name) :
        return self.__ns__[ W5_PREFIX ][name]

    #
    
    def ListServices (self,valid=None) :
        return self.__services__

    #

    def App (self) :
	subject = URIRef(NS_W5 + W5_APP)
	return self.__DescribeSubject__(subject)
    
    def DescribeService (self,service) :
        attrs = self.__DescribeSubject__(service)
        return rService(service,attrs)

    #
    
    def ListProtocols (self) :
        return self.__protocols__

    #
    
    def DescribeProtocol (self,protocol) :
        return self.__DescribeSubject__(protocol)

    #

    def SetLogging (self,bool) :
        self._SetPref("logging",bool)
    
    #

    def SetPositionX (self,pos) :
        self._SetPref("pos-x",pos)

    def SetPositionY (self,pos) :
        self._SetPref("pos-y",pos)

    #
    
    def SetMaxWidth (self, width) :
        self._SetPref("max-width",width)

    def SetMaxHeight (self, height) :
        self._SetPref("max-height",height)
        
    def _SetPref (self,which,what) :

        app   = URIRef(self.QAttr(W5_APP))
        prop  = self.QAttr(which)
        value = Literal(what)

        #
        
        self.__store__.add((app,
                            TYPE,
                            URIRef(self.QAttr(W5_PREFS))
                            ))

        #
        
        self.__store__.remove((app,prop,None))  
        self.__store__.add((app,prop,value))

    #
    
    def SetService (self,attrs) :
        
        # this is a hack - there doesn't
        # appear to be a way to modify an
        # existing set of triples in place
        
        endpoint = URIRef(attrs["endpoint"])
        
        self.__store__.remove((URIRef(endpoint),None,None))
        self.AddService(attrs)

    #
    
    def AddService (self,attrs) :
        
        endpoint = URIRef(attrs["endpoint"])

        #

        self.__store__.add((endpoint,
                            TYPE,
                            URIRef(self.QAttr(W5_SERVICE))
                            ))
        
        self.__store__.add((endpoint,
                            self.QAttr(W5_PROTOCOL),
                            URIRef(PROTOCOL_XMLRPC)
                            ))

        #
        
        if attrs.has_key("title") :
            self.__store__.add((endpoint,
                                self.__ns__[DC_PREFIX][DC_TITLE],
                                Literal(attrs["title"])
                                ))

        if attrs.has_key("login") :
            self.__store__.add((endpoint,
                                self.QAttr(W5_LOGIN),
                                Literal(attrs["login"])
                                ))

        if attrs.has_key("filter") and attrs["filter"] != None:
            self.__store__.add((endpoint,
                                self.QAttr(W5_FILTER),
                                Literal(attrs["filter"])
                                ))
            

    #

    def AddProtocol(self,uri,title):

        endpoint = URIRef(uri)
        
        self.__store__.add((endpoint,
                            TYPE,
                            URIRef(self.QAttr(W5_PROTOCOL))
                            ))

        self.__store__.add((endpoint,
                            self.__ns__[DC_PREFIX][DC_TITLE],
                            Literal(title)
                            ))

    #
    
    def RemoveService (self,endpoint) :
        
        self.__store__.remove((URIRef(endpoint),None,None))
        self.Write()
        self.Load()

    #
    
    def __DescribeSubject__ (self,subject) :

        attrs = {}

	# print "describe %s" % subject

        for p in self.__store__.predicates(subject):

	    # print "p %s" % p
            # to revisit - possibly turn it
            # in to a list or maybe just call
            # 'first_object'
            
            attrs[p] = self.__store__.objects(subject,p).next()
            
        return attrs
